using System;
using System.Collections;
using System.Drawing;
using CenterSpace.NMath.Core;
using CenterSpace.NMath.Matrix;
using System.Windows.Forms;

namespace SpectralNet
{
	/// <summary>
	/// Represents a connected component of the graph.  This is where we calculate a vertex's spectral info.
	/// See Vertex.ComponentInfo to access a vertex's spectral information.
	/// This class holds the SparseMatrix adjacency matrix.
	/// </summary>
	public class ConnectedComponent 
	{
		/// <summary>
		/// The adjacency matrix.
		/// </summary>
		SparseMatrix _adj;

		/// <summary>
		/// The vertices in this graph.  This is a duplication of data in its parent's ChemGraph, which isn't good.
		/// </summary>
		SortedList _vertices;

		/// <summary>
		/// The graph this cc belongs to.
		/// </summary>
		ChemGraph _graph;
		
		/// <summary>
		/// The shortest path matrix for this cc.  [i,j] represents the shortest path from cc.getByIndex(i) to cc.getByIndex(j).
		/// </summary>
		double[,] _shortestPath;

		/// <summary>
		/// The average diameter of this cc.
		/// </summary>
		double _avgDiameter;

		/// <summary>
		/// The URL location of this cc's laplacian LLE graph.
		/// </summary>
		string _laplacianGraphLocation;

		/// <summary>
		/// This cc's minimum average path length (across all vertices).
		/// </summary>
		double _minAvgPathLength;

		/// <summary>
		/// This cc's maximum average path length (across all vertices).
		/// </summary>
		double _maxAvgPathLength;

		/// <summary>
		/// This cc's adjacency eigenvalues.  Filled in in .calculateEigenvalues().
		/// </summary>
		public DoubleVector adjEigens;

		/// <summary>
		/// This cc's laplacian eigenvalues.  Filled in in .calculateEigenvalues().
		/// </summary>
		public DoubleVector lapEigens;

		/// <summary>
		/// This cc's normalized laplacian eigenvalues.  Filled in in .calculateEigenvalues().
		/// </summary>
		public DoubleVector normLapEigens;

		/// <summary>
		/// This cc's adjacency eigenvalues' Inverse Participation Ratio.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] adjIVP;

		/// <summary>
		/// This cc's laplacian eigenvalues' Inverse Participation Ratio.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] lapIVP;

		/// <summary>
		/// This cc's normalized laplacian eigenvalues' Inverse Participation Ratio.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] normLapIVP;

		/// <summary>
		/// These matrices hold the eigenvectors HORIZONTALLY (i.e. adjEigenvectors[0,x] is eig 1)
		/// </summary>
		public DoubleMatrix adjEigenvectors;

		/// <summary>
		/// These matrices hold the eigenvectors HORIZONTALLY (i.e. lapEigenvectors[0,x] is eig 1)
		/// </summary>
		public DoubleMatrix lapEigenvectors;

		/// <summary>
		/// These matrices hold the eigenvectors HORIZONTALLY (i.e. normLapEigenvectors[0,x] is eig 1)
		/// </summary>
		public DoubleMatrix normLapEigenvectors;

		/// <summary>
		/// Order for calculating Euler's Gamma function
		/// </summary>
		public double order = 10;

		/// <summary>
		/// The actual input to the Fermi-Dirac formula (this is a hack to get around the CentralSpace software)
		/// </summary>
		public double FermiX = 0;

		/// <summary>
		/// Property wrapper for _laplacianGraphLocation.
		/// </summary>
		public string LaplacianGraphLocation
		{
			get { return _laplacianGraphLocation; }
			set { _laplacianGraphLocation = value; }
		}

		/// <summary>
		/// Property wrapper for _minAvgPathLength.
		/// </summary>
		public double MinAveragePathLength 
		{
			get { return _minAvgPathLength; }
			set { _minAvgPathLength = value; }
		}

		/// <summary>
		/// Property wrapper for _maxAvgPathLength.
		/// </summary>
		public double MaxAveragePathLength
		{
			get { return _maxAvgPathLength; }
			set { _maxAvgPathLength = value; }
		}

		/// <summary>
		/// Property wrapper for _avgDiameter.
		/// </summary>
		public double AverageDiameter 
		{
			get { return _avgDiameter; }
		}

		/// <summary>
		/// Property wrapper for _adj.
		/// </summary>
		public SparseMatrix adj 
		{
			get { return _adj; }
		}

		/// <summary>
		/// Property wrapper for _graph.
		/// </summary>
		public ChemGraph Graph 
		{
			get { return _graph; }
			set { _graph = value; }
		}

		/// <summary>
		/// Property wrapper for _vertices
		/// </summary>
		public SortedList Vertices 
		{
			get { return _vertices; }
		}

		/// <summary>
		/// Returns the number of vertices in this cc.
		/// </summary>
		public int NumVertices 
		{
			get { return _vertices.Count; }
		}

		/// <summary>
		/// Dynamically finds the edges in this cc.
		/// </summary>
		public ArrayList Edges 
		{
			// not the best implementation, but whatever
			get
			{
				ArrayList result = new ArrayList();

				foreach (Edge e in Graph.Edges)
				{
					if (e.v1.ConnectedComponent == this)
					{
						result.Add(e);
					}
				}

				return result;
			}
		}

		/// <summary>
		/// Constructs a ConnectedComponent.
		/// </summary>
		public ConnectedComponent() 
		{
			_vertices = new SortedList();
			_minAvgPathLength = double.MaxValue;
			_maxAvgPathLength = double.MinValue;
		}

		/// <summary>
		/// Finds the minimum degree for this cc.
		/// </summary>
		public double MinDegree 
		{
			get 
			{
				double result = double.MaxValue;

				foreach(DictionaryEntry de in _vertices) 
				{
					if (((Vertex)(de.Value)).Degree < result) 
					{
						result = ((Vertex)(de.Value)).Degree;
					}
				}

				return result;
			}
		}

		/// <summary>
		/// Finds the maximum degree for this cc.
		/// </summary>
		public double MaxDegree 
		{
			get 
			{
				double result = double.MinValue;

				foreach(DictionaryEntry de in _vertices) 
				{
					if (((Vertex)(de.Value)).Degree > result) 
					{
						result = ((Vertex)(de.Value)).Degree;
					}
				}

				return result;
			}
		}

		/// <summary>
		/// Adds a vertex to this cc.
		/// </summary>
		/// <param name="v">The vertex to add.</param>
		public void addVertex(Vertex v) 
		{
			_vertices.Add(v.ID, v);
			v.ConnectedComponent = this;
		}

		/// <summary>
		/// Creates the adjacency matrix for this connected component.
		/// 
		/// NOTE that this matrix is 0-indexed and is NOT INDEXED BASED ON NODE ID.  You can reproduce the indexing scheme
		/// by a) doing a foreach on _vertices, or b) doing a .getByIndex(i) while for'ing from 0 to NumVertices
		/// </summary>
		/// <param name="size">The number of vertices in the graph, which is the rank of the adjacency matrix.</param>
		public void createAdjMatrix(int size) 
		{
			_adj = new SparseMatrix(size);

			adjIVP = new double[size];
			lapIVP = new double[size];
			normLapIVP = new double[size];

			for (int j = 0; j < size; j++)
			{
				adjIVP[j] = 0.0;
				lapIVP[j] = 0.0;
				normLapIVP[j] = 0.0;
			}

			int i = 0;
			foreach(DictionaryEntry de in _vertices) 
			{
				Vertex v = (Vertex)(de.Value);
				ArrayList conn = v.ConnectedTo;

				int j = 0;
				foreach(DictionaryEntry de2 in _vertices) 
				{
					Vertex u = (Vertex)(de2.Value);

					if (conn.Contains(u)) 
					{
						_adj.setValue(i, j, v.getEdgeWithVertex(u).weight);
					}

					j++;
				}

				i++;
			}
		}

		/// <summary>
		/// Does all the nifty calculations on this connected component.  Includes path length, eigeninformation,
		/// cluster coefficient, etc.  And it sets graph coordinates for vertices for these values.
		/// </summary>
		public void calculateVertexComponentInformation(ProgressBar pb) 
		{
			// Find the shortest path between every node
			if (Graph.options.doDijkstra == true)
			{
				findPathLengths();
			}

			foreach(DictionaryEntry de in _vertices) 
			{
				Vertex v = (Vertex)(de.Value);
				v.AdjacencyEigenInformation = new EigenInformation(this._vertices.Count);
				v.LaplacianEigenInformation = new EigenInformation(this._vertices.Count);
				v.NormalizedLaplacianEigenInformation = new EigenInformation(this._vertices.Count);
			}

			// Calculate eigenvalues
			calculateEigenValues(EigenType.Adjacency); // adjacency matrix
			if (!this.Graph.submittedViaCommandLine) pb.Value = 30;
			calculateEigenValues(EigenType.Laplacian); // laplacian
			if (!this.Graph.submittedViaCommandLine) pb.Value = 50;
			calculateEigenValues(EigenType.NormalizedLaplacian); // normalized laplacian
			if (!this.Graph.submittedViaCommandLine) pb.Value = 70;

			foreach(DictionaryEntry de in _vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (Graph.options.doClusterCoeff == true)
				{
					setClusterCoefficient(v);
				}

				// Now turn on display
				v.hasComponentInfo = true;
			}

			// Set coordinates for each vertex's cluster coefficient
			setCCCoords();
			setPathLengthCoords();
			setEigenVectorCoords(EigenType.Adjacency);
			setEigenVectorCoords(EigenType.Laplacian);
			setEigenVectorCoords(EigenType.NormalizedLaplacian);
			calculateLaplacianGraphCoords();
		}

		/// <summary>
		/// Calculates graph coordinates in the Laplacian LLE graph for each vertex.
		/// </summary>
		private void calculateLaplacianGraphCoords()
		{
			double xScale = 1.0;
			double yScale = 1.0;

			double numVertices = Vertices.Count;

			int w = this.Graph.mainDisplayWidth;
			int h = this.Graph.mainDisplayHeight;

			//
			// Set original graph positions
			foreach (DictionaryEntry de in Vertices)
			{
				Vertex v = (Vertex)de.Value;

				v.XLap = v.getLaplacianEigenvalue(1);
				v.YLap = v.getLaplacianEigenvalue(2);
			}

			//
			// Scale graph
			//
			// Find x, y scales
			double minX = Double.MaxValue;
			double maxX = Double.MinValue;
			double minY = Double.MaxValue;
			double maxY = Double.MinValue;
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v.XLap < minX) 
				{
					minX = v.XLap;
				}

				if (v.XLap > maxX) 
				{
					maxX = v.XLap;
				}

				if (v.YLap < minY) 
				{
					minY = v.YLap;
				}

				if (v.YLap > maxY) 
				{
					maxY = v.YLap;
				}
			}

			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				v.XLap -= (maxX + minX) / 2.0;
				v.YLap -= (maxY + minY) / 2.0;
			}
				
			xScale = (((double) w)*.5 - 20) / (maxX - ((maxX + minX) / 2.0));
			yScale = (((double) h)*.5 - 20) / (maxY - ((maxY + minY) / 2.0));

			// Scale vertices
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				v.XLap = v.XLap * xScale + (((double) w)*.5 - 5);
				v.YLap = v.YLap * yScale + (((double) h)*.5 - 5);
			}
		}

		/// <summary>
		/// Finds the average diameter of the graph based on shortest path information
		/// </summary>
		private void findAverageDiameter()
		{
			double numerator = 0.0;
			double denominator = (_vertices.Count)*(_vertices.Count) - _vertices.Count;

			for (int i = 0; i < _vertices.Count; i++)
			{
				for (int j = 0; j < _vertices.Count; j++)
				{
					numerator += _shortestPath[i,j];
				}
			}

			_avgDiameter = numerator / denominator;
		}

		/// <summary>
		/// This function either runs a BFS or Floyd's algorithm to determine shortest path, depending
		/// on whether the graph is weighted or unweighted
		/// </summary>
		private void findPathLengths()
		{
			// Allocate shortest path matrix
			// This is indexed as _adj is indexed, from index of _vertices
			_shortestPath = new double[_vertices.Count, _vertices.Count];

			if (this.Graph.minEdgeWeight == this.Graph.maxEdgeWeight)
			{
				//
				//
				// The unweighted case -- BFS

				//
				// REMEMBER that adjacency matrix is indexed by index of _vertices!!
				for (int i = 0; i < _vertices.Count; i++)
				{
					Vertex curVertex = (Vertex)_vertices.GetByIndex(i);

					//
					// Set up BFS starting at node curVertex
					//
					Queue q = new Queue();

					// Set every distance to infinity
					for (int j = 0; j < _vertices.Count; j++)
					{
						_shortestPath[i,j] = double.PositiveInfinity;
					}

					q.Enqueue(curVertex);
				
					// Set path from i to i to 0
					_shortestPath[i,i] = 0;

					//
					// Do the BFS
					//
					while (q.Count > 0)
					{
						Vertex u = (Vertex)q.Dequeue();
						int indexOfU = _vertices.IndexOfValue(u);

						foreach (Vertex w in u.ConnectedTo)
						{
							int indexOfW = _vertices.IndexOfValue(w);

							if (_shortestPath[i,indexOfW] == double.PositiveInfinity)
							{
								_shortestPath[i,indexOfW] = _shortestPath[i,indexOfU] + 1;
								q.Enqueue(w);
							}
						}
					}

					// set node min,max,avg info
					curVertex.MinDist = double.MaxValue;
					curVertex.MaxDist = double.MinValue;
					curVertex.AvgDist = 0.0;
					for (int j = 0; j < _vertices.Count; j++)
					{
						if ((i != j) && (_shortestPath[i,j] < curVertex.MinDist)) curVertex.MinDist = _shortestPath[i,j];
						if ((i != j) && (_shortestPath[i,j] > curVertex.MaxDist)) curVertex.MaxDist = _shortestPath[i,j];
						curVertex.AvgDist += (_shortestPath[i,j] / ((double)_vertices.Count - 1));
					}
					if (curVertex.AvgDist < this._minAvgPathLength) this._minAvgPathLength = curVertex.AvgDist;
					if (curVertex.AvgDist > this._maxAvgPathLength) this._maxAvgPathLength = curVertex.AvgDist;
				}
			}
			else
			{
				// Push the adjacency matrix into the shortest path matrix, the initiation
				// for Floyd's algorithm
				for(int i=0; i < this.NumVertices; i++)
				{
					for(int j=0; j < this.NumVertices; j++)
					{
						if (i == j) 
						{
							_shortestPath[i, j] = 0;
							continue;
						}

						_shortestPath[i, j] = this.adj.getValue(i, j);
						if (_shortestPath[i, j] == 0) _shortestPath[i, j] = double.PositiveInfinity;
					}
				}

				//
				// Floyd's algorithm
				for (int k=0; k < this.NumVertices; k++)
					for (int i=0; i < this.NumVertices; i++)
						for (int j=0; j < this.NumVertices; j++) 
						{
							if ((_shortestPath[i,k] + _shortestPath[k, j]) < _shortestPath[i, j])
								_shortestPath[i,j] = _shortestPath[i,k] + _shortestPath[k, j];
						}


				//
				// Assign dist values into vertex object
				for(int i = 0; i < _vertices.Count; i++)
				{
					Vertex s = (Vertex) _vertices.GetByIndex(i); // source vertex
					
					// set node min,max,avg info
					s.MinDist = double.PositiveInfinity;
					s.MaxDist = double.NegativeInfinity;
					s.AvgDist = 0.0;
					for (int j = 0; j < _vertices.Count; j++)
					{
						if ((i != j) && (_shortestPath[i,j] < s.MinDist)) s.MinDist = _shortestPath[i,j];
						if ((i != j) && (_shortestPath[i,j] > s.MaxDist)) s.MaxDist = _shortestPath[i,j];
						s.AvgDist += (_shortestPath[i,j] / ((double)_vertices.Count - 1));
					}
					if (s.AvgDist < this._minAvgPathLength) this._minAvgPathLength = s.AvgDist;
					if (s.AvgDist > this._maxAvgPathLength) this._maxAvgPathLength = s.AvgDist;
				}
			}

			// Set the average diameter -- MUST COME AFTER findShortestPaths()!!!
			findAverageDiameter();
		}

		/// <summary>
		/// Finds and sets the average path length graph coordinates for each vertex.
		/// </summary>
		public void setPathLengthCoords()
		{
			int w = this.Graph.smallDisplayWidth;
			int h = this.Graph.smallDisplayHeight;

			foreach(DictionaryEntry de in _vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (this.MaxDegree == this.MinDegree)
				{
					v.xAverageDistance = 30.0 + .5 * ((double)w - 60.0);
				}
				else
				{
					v.xAverageDistance = 30.0 + (1.0 - (((double)(this.MaxDegree - v.ConnectedTo.Count)) / 
						((double)(this.MaxDegree - this.MinDegree)))) * ((double)(w - 60));
				}

				if (this.MaxAveragePathLength == this.MinAveragePathLength)
				{
					v.yAverageDistance = 30.0 + .5 * ((double)h - 60.0) - 2.0;
				}
				else
				{
					v.yAverageDistance = 30.0 + (((double)(this.MaxAveragePathLength - v.AvgDist)) / 
						((double)(this.MaxAveragePathLength - this.MinAveragePathLength))) * ((double)(h - 60)) - 2.0;
				}
			}
		}

		/// <summary>
		/// Sets Y-VALUE of eigenvalue coordinates.  Doesn't set x-value because that's determined by 
		/// sort preference.
		/// </summary>
		private void setEigenVectorCoords(EigenType type) 
		{
			int w = this.Graph.smallDisplayWidth;
			int h = this.Graph.smallDisplayHeight;

			double minVal = Double.MaxValue;
			double maxVal = Double.MinValue;

			for (int i = 0; i < 4; i++) // to iterate through EigenInformation.
			{
				minVal = Double.MaxValue;
				maxVal = Double.MinValue;

				//
				// Find min/maxval
				//
				foreach(DictionaryEntry de in _vertices) 
				{
					Vertex v = (Vertex)(de.Value);
					switch (type)
					{
						case EigenType.Adjacency:
							if (v.getAdjacencyEigenvalue(i) < minVal)
								minVal = v.getAdjacencyEigenvalue(i);
							if (v.getAdjacencyEigenvalue(i) > maxVal)
								maxVal = v.getAdjacencyEigenvalue(i);
							break;
						case EigenType.Laplacian:
							if (v.getLaplacianEigenvalue(i) < minVal)
								minVal = v.getLaplacianEigenvalue(i);
							if (v.getLaplacianEigenvalue(i) > maxVal)
								maxVal = v.getLaplacianEigenvalue(i);
							break;
						case EigenType.NormalizedLaplacian:
							if (v.getNormalizedLaplacianEigenvalue(i) < minVal)
								minVal = v.getNormalizedLaplacianEigenvalue(i);
							if (v.getNormalizedLaplacianEigenvalue(i) > maxVal)
								maxVal = v.getNormalizedLaplacianEigenvalue(i);
							break;
					}
				}

				//
				// Set coordinates (Y-VALUE ONLY!!! SEE SUMMARY!)
				//
				foreach(DictionaryEntry de in _vertices)
				{
					Vertex v = (Vertex)(de.Value);
					switch (type)
					{
						case EigenType.Adjacency:
							v.AdjacencyEigenInformation.x[i] = 0;
							if ((maxVal - minVal) > 0)
							{
								v.AdjacencyEigenInformation.y[i] = 30.0 + ((maxVal - v.getAdjacencyEigenvalue(i)) 
									/ (maxVal - minVal)) * ((double)(h - 60)) - 2.0;
							}
							else
							{
								v.AdjacencyEigenInformation.y[i] = 30.0 + ((double)(h - 60))*(0.5) - 2.0;
							}
							break;
						case EigenType.Laplacian:
							v.LaplacianEigenInformation.x[i] = 0;
							if ((maxVal - minVal) > 0)
							{
								v.LaplacianEigenInformation.y[i] = 30.0 + ((maxVal - v.getLaplacianEigenvalue(i)) 
									/ (maxVal - minVal)) * ((double)(h - 60)) - 2.0;
							}
							else
							{
								v.LaplacianEigenInformation.y[i] = 30.0 + ((double)(h - 60))*(0.5) - 2.0;
							}
							break;
						case EigenType.NormalizedLaplacian:
							v.NormalizedLaplacianEigenInformation.x[i] = 0;
							if ((maxVal - minVal) > 0)
							{
								v.NormalizedLaplacianEigenInformation.y[i] = 30.0 + ((maxVal - v.getNormalizedLaplacianEigenvalue(i)) 
									/ (maxVal - minVal)) * ((double)(h - 60)) - 2.0;
							}
							else
							{
								v.NormalizedLaplacianEigenInformation.y[i] = 30.0 + ((double)(h - 60))*(0.5) - 2.0;
							}
							break;
					}
				}
			}
		}
		
		/// <summary>
		/// Sets Cluster Coefficient coordinates.
		/// </summary>
		private void setCCCoords() 
		{
			int w = this.Graph.smallDisplayWidth;
			int h = this.Graph.smallDisplayHeight;

			double minCC = 0.0;
			double maxCC = 1.0;

			foreach(DictionaryEntry de in _vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (Double.IsNaN(v.ClusterCoeff) == false) 
				{
					if (this.MaxDegree == this.MinDegree)
					{
						v.xCC = 30.0 + .5 * ((double)w - 60.0);
					}
					else
					{
						v.xCC = 30.0 + (1.0 - (((double)(this.MaxDegree - v.Degree)) / 
							((double)(this.MaxDegree - this.MinDegree)))) * ((double)(w - 60));
					}
					
					if (maxCC == minCC)
					{
						v.yCC = 30.0 + .5 * ((double)h-60) - 2.0;
					}
					else
					{
						v.yCC = 30.0 + (((double)(maxCC - v.ClusterCoeff)) / 
							((double)(maxCC - minCC))) * ((double)(h - 60)) - 2.0;
					}
				}
			}
		}

		/// <summary>
		/// Finds (and sets) the cluster coefficient for a vertex.
		/// 
		/// NOT SURE HOW TO CALCULATE THIS IN THE WEIGHTED CASE -- for now I'm "counting" all edge weights
		/// </summary>
		/// <param name="v">The vertex whose cluster coefficient to calculate.</param>
		private void setClusterCoefficient(Vertex v) 
		{
			double result = 0.0;
			int numLinks = 0;

			ArrayList neighbors = v.ConnectedTo;
			foreach(Vertex u in neighbors) 
			{
				foreach(Vertex w in u.ConnectedTo) 
				{
					//if (u == w) continue;
					if (neighbors.Contains(w)) 
					{
						numLinks++;
					}
				}
			}

			result = ((double)numLinks) / ((double)(neighbors.Count*(neighbors.Count - 1)));

			v.ClusterCoeff = result;

			if (Double.IsInfinity(v.ClusterCoeff)) v.ClusterCoeff = Double.NaN;
		}

		/// <summary>
		/// Calculates eigenvalue information for this connected component.
		/// 
		/// WARNING: This function will use MASSIVE AMOUNTS OF MEMORY for large matrices
		///          For example, a matrix of order 426 uses 1.4MB, while a matrix of order
		///          5000 uses 200MB.  This is a LOT of memory (at least in 2003).
		/// </summary>
		private void calculateEigenValues(EigenType type)
		{
			int n = _vertices.Count;

			DoubleSymmetricMatrix matrix = new DoubleSymmetricMatrix(n);

			//
			// 
			// Fill in the ap matrix (the matrix we're doing calculations on)
			for (int i = 0; i < n; i++) 
			{
				for (int j = 0; j < n; j++) 
				{
					if (type == EigenType.Adjacency) 
					{
						// ADJACENCY MATRIX
						matrix[i,j] =  _adj.getValue(i, j);
					}
					else if (type == EigenType.Laplacian) 
					{
						// LAPLACIAN
						Vertex v = (Vertex)_vertices.GetByIndex(i);
						Vertex u = (Vertex)_vertices.GetByIndex(j);

						if (i == j) 
						{
							matrix[i,j] = v.Degree;
						} 
						else if (v.ConnectedTo.Contains(u)) 
						{
							matrix[i,j] = -1.0 * v.getEdgeWithVertex(u).weight;
						} 
						else 
						{
							matrix[i,j] = 0;
						}
					}
					else if (type == EigenType.NormalizedLaplacian)
					{
						// NORMALIZED LAPLACIAN
						Vertex v = (Vertex)_vertices.GetByIndex(i);
						Vertex u = (Vertex)_vertices.GetByIndex(j);

						if (i == j) 
						{
							matrix[i,j] = 1;
						} 
						else if (v.ConnectedTo.Contains(u)) 
						{
							matrix[i,j] = - (1.0 / (Math.Sqrt(v.Degree * u.Degree)));
						} 
						else 
						{
							matrix[i,j] = 0;
						}
					}
				}
			}

			//
			//
			// Perform the decomposition
			DoubleSymEigDecomp eig = new DoubleSymEigDecomp(matrix);

			//
			// Retrieve the eigenvectors and eigenvalues
			switch (type) 
			{
				case EigenType.Adjacency:
					adjEigenvectors = eig.EigenVectors;
					adjEigens = eig.EigenValues;
					break;
				case EigenType.Laplacian:
					lapEigenvectors = eig.EigenVectors;
					lapEigens = eig.EigenValues;
					break;
				case EigenType.NormalizedLaplacian:
					normLapEigenvectors = eig.EigenVectors;
					normLapEigens = eig.EigenValues;
					break;
			}

			//
			// Truncate eigenvalues
			//
			for (int i = 0; i < n; i++)
			{
				if (type == EigenType.Adjacency) 
				{
					// ADJACENCY
					if (Math.Abs(adjEigens[i]) < 0.0000000000001)
					{
						adjEigens[i] = 0.0;
					}
				} 
				else if (type == EigenType.Laplacian)
				{
					// LAPLACIAN
					if (Math.Abs(lapEigens[i]) < 0.0000000000001)
					{
						lapEigens[i] = 0.0;
					}
				} 
				else if (type == EigenType.NormalizedLaplacian) 
				{
					// NORMALIZED LAPLACIAN
					if (Math.Abs(normLapEigens[i]) < 0.0000000000001)
					{
						normLapEigens[i] = 0.0;
					}
				}
			}

			// Calculate IVP values
			// Guide: 
			//		In the inner loop, j will be your eigenvector (no, not me), while i will be the node num to which it belongs (A[j,i]).
			//		So on the inside, we use j for which eigenvector we pair this to.  "type" is used for which eigeninformation.
			//
			for (int i = 0; i < n; i++)
			{
				for (int j = 0; j < n; j++) 
				{
					switch (type) 
					{
						case EigenType.Adjacency:
							this.adjIVP[i] += Math.Pow(adjEigenvectors[j,i], 4.0);
							break;
						case EigenType.Laplacian:
							this.lapIVP[i] += Math.Pow(lapEigenvectors[j,i], 4.0);
							break;
						case EigenType.NormalizedLaplacian:
							this.normLapIVP[i] += Math.Pow(normLapEigenvectors[j,i], 4.0);
							break;
					}
				}
			}

			for (int i = 0; i < n; i++)
			{
				switch (type) 
				{
					case EigenType.Adjacency:
						this.adjIVP[i] = -1.0 * Math.Log10(this.adjIVP[i]);
						break;
					case EigenType.Laplacian:
						this.lapIVP[i] = -1.0 * Math.Log10(this.lapIVP[i]);
						break;
					case EigenType.NormalizedLaplacian:
						this.normLapIVP[i] = -1.0 * Math.Log10(this.normLapIVP[i]);
						break;
				}
			}
		}

		/// <summary>
		/// Draws the laplacian LLE graph.
		/// </summary>
		/// <param name="curVertex">The current vertex.</param>
		/// <param name="graphDir">The graph directory to place the file in.</param>
		public Bitmap drawLaplacianGraph(Vertex curVertex) 
		{
			Pen whitePen = new Pen(Color.White, 1);
			Pen grayPen = new Pen(Color.Gray, 1);
			Pen blackPen = new Pen(Color.Black, 1);
			Pen bigBlackPen = new Pen(Color.Black, 2);
			Pen dgPen = new Pen(Color.DarkGray, 1);
			Pen pinkPen = new Pen(Color.Pink, 1);
			Pen orangePen = new Pen(Color.Orange, 2);
			Pen redPen = new Pen(Color.Red, 1);
			Brush whiteBrush = new SolidBrush(Color.White);
			Brush grayBrush = new SolidBrush(Color.Gray);
			Brush redBrush = new SolidBrush(Color.FromArgb(255, 51, 0));
			Brush greenBrush = new SolidBrush(Color.Green);
			Brush blackBrush = new SolidBrush(Color.Black);
			Brush orangeBrush = new SolidBrush(Color.Orange);
			Brush pinkBrush = new SolidBrush(Color.Pink);
			Brush darkGreenBrush = new SolidBrush(Color.DarkGreen);
			Brush siennaBrush = new SolidBrush(Color.FromArgb(0, 51, 153));

			int w = this.Graph.mainDisplayWidth;
			int h = this.Graph.mainDisplayHeight;
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			Vertex selectedVertex = curVertex;

			double minEdgeWeight = this.Graph.minEdgeWeight;
			double maxEdgeWeight = this.Graph.maxEdgeWeight;

			//
			//
			// Draw edges (some will be overdrawn later)
			if (Graph.drawOtherEdges == true)
			{
				if (minEdgeWeight == maxEdgeWeight)
				{
					//
					// Draw every line with equal darkness
					foreach(Edge e in Edges) 
					{
						if ((e.v1 != selectedVertex) && (e.v2 != selectedVertex))
						{
							pad.DrawLine(dgPen, (int)e.v1.XLap, (int)e.v1.YLap, (int)e.v2.XLap, (int)e.v2.YLap);
						}
					}
				}
				else
				{
					//
					// Scale the color such that minWeight = 204; maxWeight = 50
					// This yields: norm. distance from bottom = (w - minW) / (maxW - minW)
					//				color to use = 204 - ((w - minW) / (maxW - minW)) * 154
					foreach(Edge e in Edges) 
					{
						if ((e.v1 != selectedVertex) && (e.v2 != selectedVertex))
						{
							double rgbVal = 204.0 - (((e.weight - minEdgeWeight) / (maxEdgeWeight - minEdgeWeight)) * 154.0);
							pad.DrawLine(new Pen(new SolidBrush(Color.FromArgb((int)rgbVal,(int)rgbVal,(int)rgbVal))),
								(int)e.v1.XLap, (int)e.v1.YLap, (int)e.v2.XLap, (int)e.v2.YLap);
						}
					}
				}
			}
			
			//
			// Draw connected component vertices
			//
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v == selectedVertex) 
				{
					// Do nothing
				} 
				else if (Vertices.ContainsKey(v.ID)) 
				{
					pad.FillEllipse(siennaBrush, (int)v.XLap - 4, (int)v.YLap - 4, 8, 8);
				} 
				else 
				{
					throw new Exception("ConnectedComponent.drawLaplacianGraph: We shouldn't have other vertices to draw.");
				}
			}

			//
			//
			// Draw edges connected to selected vertex
			if (Graph.drawConnectedEdges == true)
			{
				if (minEdgeWeight == maxEdgeWeight)
				{
					foreach(Edge e in selectedVertex.InEdges) 
					{
						pad.DrawLine(orangePen, (int)e.v1.XLap, (int)e.v1.YLap, (int)e.v2.XLap, (int)e.v2.YLap);
					}
				}
				else
				{
					foreach(Edge e in selectedVertex.InEdges) 
					{
						double rVal = (double)Color.Yellow.R - (((e.weight - minEdgeWeight) / 
							(maxEdgeWeight - minEdgeWeight)) * (double)(Color.Yellow.R - Color.DarkOrange.R));
						double gVal = (double)Color.Yellow.G - (((e.weight - minEdgeWeight) / 
							(maxEdgeWeight - minEdgeWeight)) * (double)(Color.Yellow.G - Color.DarkOrange.G));
						double bVal = (double)Color.Yellow.B - (((e.weight - minEdgeWeight) / 
							(maxEdgeWeight - minEdgeWeight)) * (double)(Color.Yellow.B - Color.DarkOrange.B));
					
						pad.DrawLine(new Pen(new SolidBrush(Color.FromArgb((int)rVal, (int)gVal, (int)bVal)), 2), 
							(int)e.v1.XLap, (int)e.v1.YLap, (int)e.v2.XLap, (int)e.v2.YLap);
					}
				}
			}

			//
			// Draw text labels
			//
			foreach(DictionaryEntry de in Vertices)
			{
				Vertex v = (Vertex)de.Value;

				if (v != curVertex)
				{
					pad.DrawString(v.Label, new Font(FontFamily.GenericSansSerif, 7, FontStyle.Bold), 
						darkGreenBrush, new PointF((float)(v.XLap + 0.0), (float)(v.YLap + 0.0)));
				}
			}

			//
			// Draw connected vertices
			//
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (selectedVertex.ConnectedTo.Contains(v))
				{
					pad.FillEllipse(redBrush, (int)v.XLap - 4, (int)v.YLap - 4, 8, 8);
				}
			}
			

			//
			// Color selected vertex
			//
			PointF[] points = new PointF[3];
			points[0] = new PointF((float)curVertex.XLap, (float)curVertex.YLap - (float)(10.0f*Math.Sqrt(3)*(1.0f/3.0f)));
			points[1] = new PointF((float)curVertex.XLap - 7.0f, (float)curVertex.YLap + (float)(5.0f*Math.Sqrt(3)));
			points[2] = new PointF((float)curVertex.XLap + 7.0f, (float)curVertex.YLap + (float)(5.0f*Math.Sqrt(3)));
			pad.FillPolygon(redBrush, points);
			pad.DrawString(curVertex.Label, new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold), 
				darkGreenBrush, new PointF((float)(curVertex.XLap + 3.0), (float)(curVertex.YLap - 9.0)));

			return pic;
		}

		public void getSpectralDensityValues(EigenType which, int eigenvalueIndex, out double xVal, 
			out double yVal, out bool neededAbs)
		{
			double p, result, root;
			neededAbs = false;

			double eigenvalue;
				
			if (which == EigenType.Adjacency)
			{
				eigenvalue = this.adjEigens[eigenvalueIndex];
			}
			else if (which == EigenType.Laplacian)
			{
				eigenvalue = this.lapEigens[eigenvalueIndex];
			}
			else // which == EigenType.NormalizedLaplacian
			{
				eigenvalue = this.normLapEigens[eigenvalueIndex];
			}

			p = 0.0;
			int err = 0;
			result = 0;
			for (int j = 0; j < this.NumVertices; j++)
			{
				if (which == EigenType.Adjacency)
				{
					double diff = eigenvalue - this.adjEigens[j];
					DiracDeltaFortranConverted.fermid(-2, diff, 0.0000001, ref result, ref err);
				}
				else if (which == EigenType.Laplacian)
				{
					DiracDeltaFortranConverted.fermid(-2, (eigenvalue - this.lapEigens[j]), 0.000000001, ref result, ref err);
				}
				else // which == EigenType.NormalizedLaplacian
				{
					DiracDeltaFortranConverted.fermid(-2, (eigenvalue - this.normLapEigens[j]), 0.000000001, ref result, ref err);
				}
				
				if (err > 0)
				{
					throw new Exception("Error: err = " + err.ToString());
				}

				if (Math.Abs(result) > 0.0000000001)
				{
					p += (1.0 / ((double)this.NumVertices)) * result;
				}
			}

			if ((((double)this.NumVertices) * p * (1.0 - p)) < 0)
			{
				neededAbs = true;
			}

			root = Math.Sqrt(Math.Abs(((double)this.NumVertices) * p * (1.0 - p)));

			xVal = eigenvalue / root;
			yVal = p * root;
		}

	}

}
